/*
 * DrvI2C.c
 *
 *  Created on: 2012/12/27
 *      Author: 
 */

#include "DrvI2C.h"
#include "SpecialMacro.h"
#include "DrvREG32.h"

/**********************************************************************
 * Name     : DrvI2C_Open(uint32_t u32CRG)
 * function : enable the I2C and configure the I2C Bus clock CRG.
 * input    : u32CRG
 * output   : 0/1
 * version  : v1.1
 * date     : 13-12-4
 * modify   : change the operation of setting .
 *********************************************************************/
uint32_t DrvI2C_Open(uint32_t u32CRG)
{
  //i2c_00=0x0101;
  outpw(&I2C->CFG, (uint32_t)(0x1|0x1<<8));  //0x41000[0]=1b
  
  if(u32CRG > 0xff) return u32CRG;
  i2c_08=0xff000000+(u32CRG<<16);
  return E_SUCCESS;
}

void DrvI2C_Close(void)
{
  outpw(&I2C->CFG, (uint32_t)(0x1<<8));  //0x41000[0]=0b
}

/**********************************************************************
 * Name     : uint32_t DrvI2C_SlaveSet(^
 * function : enable slave mode and setting GC MODE and configure the data mode
 *         and set device address
 * input    : uslaveaddr uaddrbit uslave3byte gc_flag
 * output   : NO
 * version  : v1.1
 * date     : 13-12-12
 * modify   : change the operation of setting device address .
 *********************************************************************/

uint32_t DrvI2C_SlaveSet(
   uint32_t uSlaveAddr,
   E_DRVI2C_SLAVE_BIT uAddrBit,
   uint8_t uSlave3Byte,
   uint8_t GC_Flag)
{
  uint32_t temp;
  outpw(&I2C->CFG, (uint32_t)(0x1|0x1<<8));  //0x41000[0]=1b
  outpw(&I2C->ACT, (uint32_t)(0x80|(0x80<<8)));

  if(uAddrBit==E_DRVI2C_SLAVE_7BIT){
    if(uSlaveAddr>0xff)return uSlaveAddr;
    temp=inpw(&I2C->SID)&0xffffff00;
    outpw(&I2C->SID, (uint32_t)(temp|0x1|uSlaveAddr));
    outpw(&I2C->ACT, (uint32_t)((0x40<<8)));
  }else if(uAddrBit==E_DRVI2C_SLAVE_10BIT){
    if(uSlaveAddr>0x3ff)return uSlaveAddr;
    temp=inpw(&I2C->SID)&0xffff0000;
    outpw(&I2C->SID, (uint32_t)(temp|0x100|((uSlaveAddr&0x300)<<1)|((uSlaveAddr&0xff)|0x1)));
    outpw(&I2C->ACT, (uint32_t)(0x40|(0x40<<8)));
  }else
    return uAddrBit;

  if((uSlave3Byte>1)||(GC_Flag>1)) return 0xff;
  if(uSlave3Byte)
    outpw(&I2C->ACT, (uint32_t)(0x20|(0x20<<8)));
  else
    outpw(&I2C->ACT, (uint32_t)((0x20<<8)));

  if(GC_Flag)
      outpw(&I2C->CFG, (uint32_t)(0x4|(0x4<<8)));
    else
      outpw(&I2C->CFG, (uint32_t)((0x4<<8)));
   return E_SUCCESS;
}

void DrvI2C_SlaveDisable(void)
{
  outpw(&I2C->ACT, (uint32_t)(0x80<<8));
}


void DrvI2C_WriteData(uint8_t uData)
{
  outpw(&I2C->TAD, (uint32_t)(uData));
}

uint8_t DrvI2C_ReadData(void)
{
  return (inpw(&I2C->RAD)&0xff);
}

/**********************************************************************
 * Name     : DrvI2C_WriteOneData(uint8_t uData)
 * function : writ data to slaver
 * input    :  udata :0~0xff;
 * output   : no
 * version  : v1.1
 * date     : 13-12-4
 * modify   : add one operation "i2c_04=0xff01;"
 *********************************************************************/
void DrvI2C_WriteOneData(uint8_t uData)
{
  volatile unsigned int n;
  outpw(&I2C->TAD, (uint32_t)(uData));
  i2c_04=0xff01;   //0x41004
  for(n=0;n<=0x10;n++);asm("NOP");
}

void DrvI2C_Write3ByteData(uint8_t uData1,uint8_t uData2,uint8_t uData3)
{
  outpw(&I2C->TAD, (uint32_t)(uData1|(uData2<<8)|(uData3<<16)));
}

/**********************************************************************
 * Name     : DrvI2C_ReadOneData()
 * function : read data from slaver and sent ACK to slaver
 * input    : uack choose to answer ACK or not
 *            0 NO ACK
 *            1 ACK
 * output   : data from slaver
 * version  : v1.1
 * date     : 13-12-4
 * modify   : change operation on read data from slaver
 *            2014-8-18 קơAK[i2c_04=0xff00;
 *********************************************************************/
uint8_t DrvI2C_ReadOneData(uint8_t uack)
{
  unsigned char RecData;
  unsigned int i;
  RecData = inpw(&I2C->RAD)&0xff;
  i2c_04=0xff00+uack;
  for(i=0;i<0xffff;i++)
  {
    if(i2c_04&0x00000002)
    {
      i2c_04=0xff00;   //0x41004
      return i2c_10&0xff;
    }
  }
  return E_FAIL;
}

void DrvI2C_Ctrl(uint8_t start, uint8_t stop, uint8_t intFlag, uint8_t ack)
{
  uint32_t Reg = 0;

  if (start)
    Reg |= I2C_STA;
  if (stop)
      Reg |= I2C_STO;
  if (intFlag)
    Reg |= I2C_SI;
  if (ack)
    Reg |= I2C_AA;
  outpw(&I2C->ACT, (uint32_t)(Reg|(0xf<<8)));
}

/**********************************************************************
 * Name     : void DrvI2C_EnableInt(E_DRVI2C_INTERRUPT_SET uINT)
 * function : enable the I2C all interrupt
 * input    : uINT
 *          E_DRVI2C_INT_ENABLE:       enable the i2c interrupt
 *          E_DRVI2C_ERROR_INT_ENABLE: enable the i2c error interrupt
 *          E_DRVI2C_INT_ALL_ENABLE:   enable the i2c interrupt and error interrupt
 * output   : NO
 * version  : v1.1
 * date     : 13-12-10
 * modify   : change the operation of setting .
 *********************************************************************/
void DrvI2C_EnableInt(E_DRVI2C_INTERRUPT_SET uINT)
{
  outpw(&INT_COM->COM, (uint32_t)((uINT+1)<<20)|((1+uINT)<<28));
}

/**********************************************************************
 * Name     : void DrvI2C_DisableInt(E_DRVI2C_INTERRUPT_SET uINT)
 * function : disable the I2C all interrupt
 * input    : uINT
 *          E_DRVI2C_INT_ENABLE:       enable the i2c interrupt
 *          E_DRVI2C_ERROR_INT_ENABLE: enable the i2c error interrupt
 *          E_DRVI2C_INT_ALL_ENABLE:   enable the i2c interrupt and error interrupt
 * output   : no
 * version  : v1.1
 * date     : 13-12-10
 * modify   : change the operation of setting .
 *********************************************************************/
void DrvI2C_DisableInt(E_DRVI2C_INTERRUPT_SET uINT)
{
  outpw(&INT_COM->COM, (uint32_t)((uINT+1)<<28));
}

/**********************************************************************
 * Name     : E_DRVI2C_INTERRUPT DrvI2C_ReadIntFlag(void)
 * function : disable the I2C all interrupt
 * input    : uINT
 *          E_DRVI2C_INT:       i2c interrupt flag =1
 *          E_DRVI2C_ERROR_INT: i2c error interrupt flag =1
 *          E_DRVI2C_INT_ALL:   i2c interrupt and error interrupt flag =1
 * output   : no
 * version  : v1.1
 * date     : 13-12-10
 * modify   : change the operation of setting .
 *********************************************************************/
E_DRVI2C_INTERRUPT DrvI2C_ReadIntFlag(void)
{
  return (inpw(&INT_COM->COM)&0x30)>>4;
}

/**********************************************************************
 * Name     : void DrvI2C_ClearIntFlag(E_DRVI2C_INTERRUPT_SET uINT)
 * function : clear the I2C interrupt flag
 * input    : uINT:
 *            E_DRVI2C_INT_ENABLE:         clear interrupt flag;
 *            E_DRVI2C_ERROR_INT_ENABLE:   clear error interrupt flag
 *            E_DRVI2C_INT_ALL_ENABLE:     clear interrupt and error interrupt flag
 * output   : NO
 * version  : v1.1
 * date     : 2013-12-10
 * modify   : change the operation of setting .
 *********************************************************************/
void DrvI2C_ClearIntFlag(E_DRVI2C_INTERRUPT_SET uINT)
{
  i2c_04=0x0f01&i2c_04; //A/NA=1
  outpw(&INT_COM->COM, (uint32_t)((1+uINT)<<12));
}

void DrvI2C_ClearEIRQ(void)
{
  outpw(&I2C->ACT, (uint32_t)(0x10<<8));
}

void DrvI2C_ClearIRQ(void)
{
  outpw(&I2C->ACT, (uint32_t)(0x2<<8));
}

/*******************************************************************
*name    : uint8_t DrvI2C_GetStatusFlag(void)
*function: ŪIICAлx
*input   : NO
*output  : ^Ȧsbit[23:16]:0xff~0
*Version : v1.1
*date    : 2014-3-24
*modify  : קX^
*********************************************************************/
uint8_t DrvI2C_GetStatusFlag(void)
{
  return ((inpw(&I2C->ACT)&0xff0000)>>16);
}

uint8_t DrvI2C_TimeOutEnable(
E_DRVI2C_TIMEOUT_PRESCALE uPreScale,
E_DRVI2C_TIMEOUT_LIMIT uTimeOutLimit
)
{
  if((uPreScale>7)||(uTimeOutLimit>15)) return 0xff;
  outpw(&I2C->CTCR, (uint32_t)((uPreScale<<4)|(uPreScale<<12)|(uTimeOutLimit)|(uTimeOutLimit<<8)));
  outpw(&I2C->CFG, (uint32_t)(0x2|(0x2<<8)));
  return E_SUCCESS;
}

void DrvI2C_TimeOutDisable(void)
{
  outpw(&I2C->CFG, (uint32_t)(0x2<<8));
}

/**********************************************************************
 * Name     : DrvI2C_SetIOPin()
 * function : enable and setting the IO port for I2C
 * input    : upin choose the io port
 *            0  SCL=PT1.0;SDA=PT1.1
 *            1  SCL=PT1.2;SDA=PT1.3
 *            2  SCL=PT1.4;SDA=PT1.5
 *            3  SCL=PT1.6;SDA=PT1.7
 *            4  SCL=PT2.0;SDA=PT2.1
 *            5  SCL=PT2.2;SDA=PT2.3
 *            6  SCL=PT2.4;SDA=PT2.5
 *            7  SCL=PT2.6;SDA=PT2.7
 * output   : 0/1
 *            0 setting fail
 *            1 setting success
 * version  : v1.0
 * date     : 13-12-4
 * modify   : no
 *********************************************************************/
unsigned char DrvI2C_SetIOPin(unsigned int upin)
{
  signed int l_reg,l_data;
  l_reg=R_I2C_IO;
  if(upin<8)
  {
    l_data=0x10000+(upin<<17)+0x0f000000;
    outw(l_reg,l_data);
    return E_SUCCESS;
  }
  else
    return E_FAIL;
}

/*******************************************************************
*name    : DrvI2C_DisableIOPin()
*function: IO_άI2CqTIO\
*input   : NONE
*output  : NONE
*Version : v1.0
*date    : 2014-1-17
*modify  : NONE
*********************************************************************/
void DrvI2C_DisableIOPin(void)
{
  pioc_2=0x01000000;
}

/**********************************************************************
 * Name     : void DrvI2C_STSP(unsigned char usignal)
 * function : I2C start/stop signal
 * input    : usignal : 0 IIC START signal
 *                      1 IIC STOP signal
 * output   : no
 * version  : v1.0
 * date     : 13-12-4
 * modify   : 2014-8-18  קơAK[i2c_04=0xf300;Aקfor`
 *********************************************************************/
void DrvI2C_STSP(unsigned char usignal)
{
  uint32_t Reg = 0;

  if (usignal==0)
    Reg |= I2C_STA;
  if (usignal==1)
      Reg |= I2C_STO;
  outpw(&I2C->ACT, (uint32_t)(Reg|(0xf<<8)));

  i2c_04=0xf300;   //0x41004
  volatile unsigned int n;
  for(n=0;n<=0x50;n++);//asm("NOP");
}

/*******************************************************************
*name    : DrvI2C_MGetACK()
*function: Slaver answer the ACK to Master,used for master transmit data to slaver
*input   : utime  for check time is 0~0xffff
*output  : 0 success
*          1 fail
*Version : v1.0
*date    : 2013-12-4
*modify  : 2014-8-18 K[i2c_04=0xff00;
*********************************************************************/
unsigned char DrvI2C_MGetACK(unsigned int utime)
{
    unsigned int i;
    unsigned char Returnvalue;
    Returnvalue=E_FAIL;
    for(i=0;i<utime;i++)
    {
        if(i2c_04&0x00000002)
        {
           Returnvalue=E_SUCCESS;
           i2c_04=0xff00;   //0x41004
           break;
        }
    }
    return Returnvalue;
}
/****************************************************************************
*name    : DrvI2C_ClkEnable();
*function: setting the I2C clock source and setting divier;
*input   : uclk: clock source
*              0: external high speed osc
*              1: internal high speed osc
*          uprescale: clock divier;
*output  : 
*version : v1.0
*date    : 2014-5-30
*modify  : no
****************************************************************************/
unsigned int DrvI2C_ClkEnable(unsigned int uclk)
{
  unsigned int l_reg,l_data;
  l_reg=R_I2C_CLOCK;

  l_data=inw(l_reg)&0xfffeffff;
  if(uclk>1) {return E_FAIL;}
  if(uclk==0)
  {
    CLRBIT(l_data,I2CCKS);
  }
  else
  {
    SETBIT(l_data,I2CCKS);
  }
  l_data=l_data+0x01000000;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/****************************************************************************
*name    : DrvI2C_ResetI2C_Reset();
*function: Reset I2C;
*input   : 
*output  : 
*version : v1.0
*date    : 2021-08-18
*modify  : no
****************************************************************************/
void DrvI2C_Reset(void)
{
	outpw(&I2C->CFG, (uint32_t)(0x1<<8));  //I2C Close, 0x41000[0]=0b
	DrvI2C_ClearIRQ();
	DrvI2C_ClearEIRQ();            // Clear EIRQFlag
	DrvI2C_ClearIntFlag(1);        // Clear I2C Interrupt Flag(I2CEIF)
	outpw(&I2C->CFG, (uint32_t)(0x1|0x1<<8)); //I2C Open, 0x41000[0]=1b
	DrvI2C_Ctrl(0,0,0,0);          // Clear all I2C flag
}