/*
 *  DrvLCD.c
 *
 *  Created on: 2014/5/31
 *  Author:
 */
#include "DrvLCD.h"
#include "SpecialMacro.h"

/**********************************************************************************
*NAME  : DrvLCD_EnableCLK()
*FUNCT : SETTING LCD1 AND LCD2 CLOCK
*INPUT :
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_EnableCLK(unsigned int uLCD1,unsigned int uLCD2,unsigned int usource)
{
  unsigned int l_reg,l_data;
  if((uLCD1>LCDO_MAX)||(uLCD2>LCDE_MAX)||(usource>1))
    return E_FAIL;
  l_data=(uLCD1<<LCDO)|(LCDO_MAX<<LCDO_MASK)|(uLCD2<<LCDE)|(LCDE_MAX<<LCDE_MASK)|(usource<<LCKS)|(LCKS_MASK<<LCKS_MASK);
  l_reg=LCD_CLOCK_CTRL;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_LCDEnable()
*FUNCT : LCD Enable
*INPUT : UDISMODE
*        0 Disable  ; 1 Enable
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LCDEnable(unsigned int umode)
{
  unsigned int l_reg,l_data;
  if(umode>1)
    return E_FAIL;
  l_data=(umode<<LCDEN)|(1<<LCDEN_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_DisplayMode()
*FUNCT : SETTING LCD DISPLAY MODE
*INPUT : UDISMODE
*        0 NORMAL ; 1 ALL PIXELS ON; 2 ALL PIXELS OFF; 3 NORMAL MODE
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_DisplayMode(E_LCD_DISPLAY_MODE uDISMODE)
{
  unsigned int l_reg,l_data;
  if(uDISMODE>DSP_MAX)
    return E_FAIL;
  l_data=(uDISMODE<<DSP)|(DSP_MAX<<DSP_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_VLCDMode()
*FUNCT : SETTING VLCD MODE
*INPUT : UVLCDMODE
*        0: Rev; 1: Rev; 2: 2.8V; 3: 3.0V; 4: 3.3V; 5: 3.94V; 6: 4.5V; 7: 5.0V
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.2
*DATE  : 2017-12-11
*MODIFY:
*
*********************************************************************************/
unsigned char DrvLCD_VLCDMode(E_VLCD_MODE uVLCDMODE)
{
  unsigned int l_reg,l_data;
  if(uVLCDMODE<2)
    return E_FAIL;
  if(uVLCDMODE>VLCD_MAX)
    return E_FAIL;

  l_data=(uVLCDMODE<<VLCD)|(VLCD_MAX<<VLCD_MASK)|(1<<VLCDEN)|(1<<VLCDEN_MASK);

  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);

  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_VLCDEnable()
*FUNCT : LCD Enable
*INPUT : UDISMODE
*        0 Disable  ; 1 Enable
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_VLCDEnable(unsigned int umode)
{
  unsigned int l_reg,l_data;
  if(umode>1)
    return E_FAIL;
  l_data=(umode<<VLCDEN)|(1<<VLCDEN_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_LCDBias()
*FUNCT : LCD Bias Mode
*INPUT : ubias
*        0: 1/3Bias  ; 1: 1/4Bias 
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2021-4-15
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LCDBias(E_LCD_BIAS ubias)
{
  unsigned int l_reg,l_data;
  if(ubias>1)
    return E_FAIL;
  l_data=(ubias<<LCDBS)|(1<<LCDBS_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_LcdDuty()
*FUNCT : SETTING LCD DUTY MODE
*INPUT : UDUTY
*        0 1/3 DUTY ; 1 1/4 DUTY; 2  1/5 DUTY; 3  1/6 DUTY; 4 1/7 Duty; 5 1/8 Duty
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LcdDuty(E_LCD_DUTY uDUTY)
{
  unsigned int l_reg,l_data;
  if (uDUTY>DUTY_MAX)
    return E_FAIL;
  l_data=(uDUTY<<DUTY)|(DUTY_MAX<<DUTY_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_LCDBuffer()
*FUNCT : SETTING LCD OUTPUT BUFFER BEING ON OR OFF
*INPUT : UBEN
*        0 DISABLE ; 1 ENABLE;
*
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_LCDBuffer(unsigned int uBEN)
{
  unsigned int l_reg,l_data;
  if(uBEN>1)
    return E_FAIL;
  l_data=(uBEN<<BEN)|(1<<BEN_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_IOMode()
*FUNCT : SETTING THE IO port as the LCD MODE OR GPIO MODE;
*INPUT : uport
*        0 PT6LEN ; 1 PT7LEN; 2  PT8LEN; 3 PT9LEN; 4 PT10LEN; 5 PT13LEN;
*        uIOMODE :0~0xFF,bit operation
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2015-6-9
*MODIFY:
*********************************************************************************/
unsigned char DrvLCD_IOMode(E_LCD_IO_PORT uport,unsigned int uIOMODE)
{
  unsigned int l_reg,l_data;
  if(uport>5)
    return E_FAIL;
  else if(uport<4)
  {
    l_data=inw(LCD_CTRL1)&(0xFFFFFFFF^(0xFF<<(uport*8)));
    l_data=((uIOMODE<<(uport*8))|l_data); //(inw(LCD_CTRL1)&0xFFFFFFFF));
    l_reg=LCD_CTRL1;
  }
  else if(uport==4)
  {
    l_data=(inw(LCD_CTRL2)&0xFFFFFF00)|uIOMODE;
    l_reg=LCD_CTRL2;
  }
  else
  {
    l_data=(inw(LCD_CTRL2)&0x00FFFFFF)|(uIOMODE<<24);
    l_reg=LCD_CTRL2;
  }
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**********************************************************************************
*NAME  : DrvLCD_WriteData()
*FUNCT : write data to SEG
*INPUT : uSEG  LCD seg LCD0~LCD21
*        PT6:
*             PT6.0~1 LCD1
*             PT6.2~3 LCD2
*             PT6.4~5 LCD3
*             PT6.6~7 LCD4
*        PT7:
*             PT7.0~1 LCD5
*             PT7.2~3 LCD6
*             PT7.4~5 LCD7
*             PT7.6~7 LCD8
*        PT8:
*             PT8.0~1 LCD9
*             PT8.2~3 LCD10
*             PT8.4~5 LCD11
*             PT8.6~7 LCD12
*        PT9:

*             PT9.0~1 LCD13
*             PT9.2~3 LCD14
*             PT9.4~5 LCD15
*             PT9.6~7 LCD16
*        PT10:
*             PT10.0~1 LCD17
*             PT10.2~3 LCD18
*             PT10.4~5 LCD19
*             PT10.6~7 LCD20
*        PT13:
*             PT13.4~5 LCD0
*             PT13.6~7 LCD21
*
*        data : the data write to seg
*OUTPUT: 0 SUCCESS; 1 FAIL;
*VER   : V1.0
*DATE  : 2014-6-4
*MODIFY:
*
*********************************************************************************/
unsigned char DrvLCD_WriteData(unsigned int uSEG,unsigned int data)
{
  unsigned int l_reg,l_data,l_temp,l_duty;

  if(uSEG>LCD_BUFFER)
    return E_FAIL;
  l_temp=(((inw(LCD_CTRL0)>>DUTY)&0x07)+3);
  l_duty=0xFF>>(8-l_temp);
  if(uSEG==21) //PT13
  {
    l_data=(inw(LCD_CTRL2)>>24)&0x3;
  }
  else if(uSEG>=17)  //PT10
  {
    l_data=(inw(LCD_CTRL2)>>((uSEG-1)*2))&0x3;
  }
  else if(uSEG>0)  //about PT6~PT9
  {
    l_data=(inw(LCD_CTRL1)>>((uSEG-1)*2))&0x3;
  }
  else if(uSEG==0)
  {
    l_data=0x3;
  }
    if(l_data==0x3)
    {
      l_data=0xFF00FF00;
    }
    else if(l_data==0x2)
    {
      l_data=0xFF000000;
    }
    else if(l_data==0x01)
    {
      l_data=0x0000FF00;
    }
    else if(l_data==0)
      l_data=0x0;
  //l_data=((data&0x03f)<<16)|((data&0xf0)>>l_temp)|0xFF00FF00;
  l_data=((data&l_duty)<<16)|((data>>l_temp)&l_duty)|l_data;
  if(uSEG==0)
    l_reg=GPIO_LCD0;
  else if(uSEG==21)
    l_reg=GPIO_LCD21;
  else if(uSEG<21)
    l_reg=GPIO_LCD1+(uSEG-1)*0x04;
  outw(l_reg,l_data);
  return E_SUCCESS;
}

/**************************************************************
name    : unsigned char DrvLCD_VLCDTrim(short Umode)
function: qL]mAΤiHܤPVLCDqեF
input:   umode:    1: VLCD~3.43V
                   2: VLCD~3.16V
                   3: VLCD~2.93V
                   4: VLCD~2.73V
                   5: VLCD~2.55V
output :  0 SUCCESS/1 FAIL
note   G   {Ǿާ@覡G

date   :   2014-10-17
modify :

**************************************************************/
unsigned char DrvLCD_VLCDTrim(short Umode)
{
  unsigned int l_reg,l_enable,l_data;
  unsigned char l_flag;
  if((Umode==0)||(Umode>5))
    return E_FAIL;
  if((inw(0x40104)&0x00FF0000)==0x00c00000)
    l_flag=1;
  else if((inw(0x40104)&0x00FF0000)==0x001c0000)
    l_flag=2;
  switch(Umode)
  {
  case 5 :       //else if(Umode==5)
      l_data=inw(VLCD_TRIM)&0x07f;
      break;

  case 4 :      //else if(Umode==4){
    if(l_flag==1)
      l_data=(inw(0x41f00)&0x07f00)>>8;
    else if(l_flag==2)
      l_data=(inw(0x42740)&0x07f00)>>8;
    break;

  case 3 :     //else if(Umode==3){
    if(l_flag==1)
      l_data=inw(0x41f00)&0x07f;
    else if(l_flag==2)
      l_data=inw(0x42740)&0x07f;
    break;

  case 2 :    //else if(Umode==2)
    if(l_flag==1)
      l_data=(inw(0x41f04)&0x07f00)>>8;
    else if(l_flag==2)
      l_data=(inw(0x42740)&0x07f000000)>>24;
    break;

  case 1 :   //else if(Umode==1)
    if(l_flag==1)
      l_data=inw(0x41f04)&0x07f;
    else if(l_flag==2)
      l_data=(inw(0x42740)&0x07f0000)>>16;
    break;
  default :
    break;
  }
  l_enable=0x0f00+((l_data>>3)&0xf);
  outw(0x41F24,l_enable);

  l_data=l_data&0x07;
  l_data=(l_data<<VLCD)|(VLCD_MAX<<VLCD_MASK);
  l_reg=LCD_CTRL0;
  outw(l_reg,l_data);

  return E_SUCCESS;
}
