/*
 * DrvPMU.c
 *
 *  Created on: 2011/11/28
 *      Author: 
 */

#include "DrvPMU.h"
#include "DrvADC.h"
#include "DrvClock.h"
#include "SpecialMacro.h"

unsigned int DrvPMU_VDD15Trim(void)
{
  unsigned int l_reg,l_data;

  l_data= inw(Trim_VDD15_ADDR);
  if(l_data != 0xFFFFFFFF)
  {
    l_reg=PMU_CTRL;
    outw(l_reg,l_data);
  }
}
unsigned int DrvPMU_VDDA_Voltage(unsigned int uVoltage)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  if(uVoltage<(VDAS_MAX+1))
  {
    l_data=(uVoltage<<VDAS)|(VDAS_MAX<<(VDAS_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_VDDA_LDO_Ctrl(unsigned int uCtrl)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  if(uCtrl<(ENVA_MAX+1))
  {
    l_data=(uCtrl<<ENVA)|(ENVA_MAX<<(ENVA_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

void DrvPMU_BandgapEnable(void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  l_data=(1<<ENBGR)|(1<<ENBGR_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_BandgapDisable(void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  l_data=(1<<ENBGR_MASK);
  outw(l_reg,l_data);
}
/*
void DrvPMU_ChargePumpEnable(void)
{
  unsigned int l_reg,l_data;
  l_reg=PMU_CTRL;
  l_data=(1<<ENCHP)|(1<<ENCHP_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_ChargePumpDisable(void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  l_data=(1<<ENCHP_MASK);
  outw(l_reg,l_data);
}
*/
void DrvPMU_REFO_Enable(void)
{
  unsigned int l_reg,l_data;
  l_reg=PMU_CTRL;
  l_data=(1<<ENRFO)|(1<<ENRFO_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_REFO_Disable(void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  l_data=(1<<ENRFO_MASK);
  outw(l_reg,l_data);
}

unsigned int DrvPMU_AnalogGround(unsigned int uAG)
{
  unsigned int l_reg,l_data;

  l_reg=R_ADC_CTRL1;
  if(uAG<(ACMS_MAX+1))
  {
    l_data=(uAG<<ACMS)|(ACMS_MAX<<(ACMS_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_LDO_LowPower(unsigned int uLP)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL;
  if(uLP<(V15LP_MAX+1))
  {
    l_data=(uLP<<V15LP)|(V15LP_MAX<<(V15LP_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

/****************************************************************************
 * name    : DrvPMU_ReadLVDO(void)
 * function: read LVD Output
 * input   : NO
 * output  : ^LVDA
 *           0 tݹq>ݹq
 *           1 ݹq>tݹq
 *
 * version :V01
 * date    : 2020-9-25
 * modify  : 
 ****************************************************************************/
unsigned int DrvPMU_GetLVDO(void)
{
  unsigned int l_reg,ReturnVal;
  ReturnVal=0;
  l_reg=PMU_CTRL1;
  ReturnVal=(inw(PMU_CTRL1)&(1<<LVDO))>>LVDO;
  return ReturnVal;
}


void DrvPMU_EnableENLVD (void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  l_data=(1<<ENLVD)|(1<<ENLVD_MASK);
  outw(l_reg,l_data);
  
}

void DrvPMU_DisableENLVD (void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  l_data=(1<<ENLVD_MASK);
  outw(l_reg,l_data);
}

unsigned int DrvPMU_SetLVDVS(unsigned char uLVDVS)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  if(uLVDVS<2)
  {
    l_data=(uLVDVS<<LVDVS)|(1<<(LVDVS_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_SetLVD12(unsigned char uLVD12)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  if(uLVD12<2)
  {
    l_data=(uLVD12<<LVD12)|(1<<(LVD12_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;  
}

/**********************************************************************************
*NAME  : DrvPMU_SetLVDS()
*FUNCT : SETTING LVD Voltage
*INPUT : uVoltage
*        0: OFF; 1: 2.0V; 2: 2.1V; 3: 2.2V; 4: 2.3V; 5: 2.4V; 6: 2.5V; 7: 2.6V
*        8: 2.7V; 9: 2.8V; 10: 2.9V; 11: 3.0V; 12: 3.3V; 13: 3.6V; 14: 4.0V; 15: LVDIN
*
*OUTPUT: 0 SUCCESS; 1 FAIL
*VER   : V1.0
*DATE  : 2020-09-25
*MODIFY:
*
*********************************************************************************/
unsigned int DrvPMU_SetLVDS(E_LVD_LVDS uVoltage)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  if(uVoltage<(LVDS_MAX+1))
  {
    l_data=(uVoltage<<LVDS)|(LVDS_MAX<<(LVDS_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_LVDIntTriMode(E_LVDO_Status uLVDITT)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL1;
  if(uLVDITT<2)
  {
    l_data=(uLVDITT<<LVDITT)|(1<<(LVDITT_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

void DrvPMU_EnableLVDInt(void)
{
  uint32_t l_reg,l_data;
  l_reg=LVD_INTERRUPT_CTRL;
  l_data=(1<<LVDIE)|(1<<LVDIE_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_DisableLVDInt(void)
{
  uint32_t l_reg,l_data;
  l_reg=LVD_INTERRUPT_CTRL;
  l_data=(1<<LVDIE_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_ClearLVDIF(void)
{
  unsigned int l_reg,l_data;

  l_reg=LVD_INTERRUPT_CTRL;
  l_data=(1<<LVDIF_MASK);
  outw(l_reg,l_data);
}

void DrvPMU_EnableBOR2(unsigned char utime)
{
  unsigned int l_reg,l_data,l_temp,l_clk,l_count;
  l_reg=PMU_CTRL2;
  l_temp= (inw(l_reg)&(1<<BOR2S));  //backup BOR2S
  outw(l_reg,0x0B00);  //BOR2SLP=0,BOR2S=0
  l_data=(1<<BOR2EN)|(1<<BOR2EN_MASK);
  outw(l_reg,l_data);  //BOR2EN=1
  if((inw(R_CLOCK_CTRL2)&0x01)==0)  //MCUCK=HS_CK
  {
    l_count=16;  //default 4M/8, count=7, about 120us

    //check HAOM=4M or 32M
    l_clk=inw(R_CLOCK_CTRL1)&(HAMO_MAX<<HAMO);
    if(l_clk ==(E_HAO_32M<<HAMO))
    {  l_count= l_count*8;}       //32M=4M*8 => count*8

    //Check ENMCD=/2 /4 /8 /1
    l_clk=inw(R_CLOCK_CTRL2)&(ENMCD_MAX<<ENMCD);
    switch((l_clk>>ENMCD))
    {
      case 0: // MCU Clock/2
    	l_count= l_count*4;
      break;
      case 1: // MCU Clock/4
    	l_count= l_count*2;
      break;
      case 2: // MCU Clock/8
    	l_count= l_count*1;
      break;
      case 3: // MCU Clock/1
    	l_count= l_count*8;
      break;
    }

  }
  else                                 //MCUCK=LS_CK
  {
    l_count=1;
  }
  l_count=l_count*(utime+1);
  for(;l_count>0;l_count--)  //Delay >100us
    asm("NOP");
  outw(l_reg,(l_temp|(1<<BOR2S_MASK)));  //reload BOR2S
}

void DrvPMU_DisableBOR2 (void)
{
  unsigned int l_reg,l_data;

  l_reg=PMU_CTRL2;
  l_data=(1<<BOR2EN_MASK);
  outw(l_reg,l_data);
}

unsigned int DrvPMU_BOR2_Mode(E_BOR2_Mode uBOR2S)
{
  unsigned int l_reg,l_data;
  l_reg=PMU_CTRL2;
  if(uBOR2S<2)
  {
    l_data=(uBOR2S<<BOR2S)|(1<<(BOR2S_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_BOR2_DetectVol(E_BOR2_BOR2TH uBOR2TH)
{
  unsigned int l_reg,l_data;
  l_reg=PMU_CTRL2;
  if(uBOR2TH<(BOR2TH_MAX+1))
  {
    l_data=(uBOR2TH<<BOR2TH)|(BOR2TH_MAX<<(BOR2TH_MASK));
    outw(l_reg,l_data);
  }else return E_DRVPMU_ARGUMENT;
  return 0;
}

unsigned int DrvPMU_ReadBOR2Status(void)
{
  unsigned int l_reg,ReturnVal;
  ReturnVal=0;
  l_reg=PMU_CTRL2;
  ReturnVal=(inw(PMU_CTRL2)&(1<<BOR2LV))>>BOR2LV;
  return ReturnVal;
}
