/*
 * DrvPMU.h
 *
 *  Created on: 2011/11/28
 *      Author: 
 */

#ifndef DRVPMU_H_
#define DRVPMU_H_

#include "HY16F391.h"


#ifndef E_FAIL
#define E_FAIL    1
#endif
#define E_DRVPMU_ARGUMENT E_FAIL

#ifndef E_SUCCESS
#define E_SUCCESS    0
#endif

#ifdef PMU_BASE
#define PMU_CTRL      (PMU_BASE+OFFSET00)
#define PMU_CTRL1     (PMU_BASE+OFFSET08)
#define PMU_CTRL2     (PMU_BASE+OFFSET0C)

#define Trim_VDD15_ADDR    0xB1818
#endif

#ifdef  INTERRUPT_BASE
#define LVD_INTERRUPT_CTRL  (INTERRUPT_BASE+OFFSET0C)
#endif
#define LVDIE    16
#define LVDIE_MASK      (LVDIE+MASK)
#define LVDIF    0
#define LVDIF_MASK      (LVDIF+MASK)

typedef enum
{
  E_VDDA2_4 = 0,
  E_VDDA2_6 = 1,
  E_VDDA2_9 = 2,
  E_VDDA3_2 = 3          //@VDD5V=3.5V
} E_VDDA_OUTPUT_VOLTAGE;

typedef enum
{
  E_HighZ     = 0,
  E_LDO      = 1
} E_VDDA_LDO_ENABLE_CONTROL;

typedef enum
{
	LVD_V12_BOR = 0,  //0
	LVD_V12_BGR,      //1
} E_LVD_LVD12;

typedef enum
{
	LVD_VDD5V = 0,  //0
	LVD_VLCD,       //1
} E_LVD_LVDVS;

typedef enum
{
	LVD_OFF = 0,
	LVD_20V   = 1,
	LVD_21V   = 2,
	LVD_22V   = 3,
	LVD_23V   = 4,
	LVD_24V   = 5,
	LVD_25V   = 6,	
	LVD_26V   = 7,
	LVD_27V   = 8,
	LVD_28V   = 9,
	LVD_29V   = 10,
	LVD_30V   = 11,
	LVD_33V   = 12,		
	LVD_36V   = 13,
	LVD_40V   = 14,
	LVD_LVDIN = 15,
} E_LVD_LVDS;


typedef enum
{
  E_NGP = 0,
  E_PGN,
} E_LVDO_Status;

typedef enum
{
	BOR2_17V = 0,
	BOR2_20V   = 1,
	BOR2_22V   = 2,
	BOR2_25V   = 3,	
	BOR2_27V   = 4,
	BOR2_30V   = 5,	
	BOR2_36V   = 6,
	BOR2_40V   = 7
} E_BOR2_BOR2TH;

typedef enum
{
  E_Inturrept = 0,
  E_Reset=1
} E_BOR2_Mode;

//PPMU_CTRL
#define VDAS            18
#define ENVA            16
#define ENBGR           4
//#define ENCHP           2
#define ENRFO           1
#define V15LP           0
#define VDAS_MASK       (VDAS+MASK)
#define ENVA_MASK       (ENVA+MASK)
#define ENBGR_MASK      (ENBGR+MASK)
//#define ENCHP_MASK      (ENCHP+MASK)
#define ENRFO_MASK      (ENRFO+MASK)
#define V15LP_MASK      (V15LP+MASK)
#define VDAS_MAX           3
#define ENVA_MAX           1
#define V15LP_MAX          1

//PMU_CTRL1
#define LVDO           16
#define LVDS           4
#define LVDITT         3
#define LVD12          2
#define LVDVS          1
#define ENLVD          0
#define LVDO_MASK       (LVDO+MASK)
#define LVDS_MASK       (LVDS+MASK)
#define LVDITT_MASK     (LVDITT+MASK)
#define LVD12_MASK      (LVD12+MASK)
#define LVDVS_MASK      (LVDVS+MASK)
#define ENLVD_MASK      (ENLVD+MASK)
#define LVDS_MAX           15

//PMU_CTRL2
#define BOR2TH         4
#define BOR2SLP        3
#define BOR2LV         2
#define BOR2S          1
#define BOR2EN         0
#define BOR2TH_MASK     (BOR2TH+MASK)
#define BOR2SLP_MASK    (BOR2SLP+MASK)
#define BOR2LV_MASK     (BOR2LV+MASK)
#define BOR2S_MASK      (BOR2S+MASK)
#define BOR2EN_MASK     (BOR2EN+MASK)
#define BOR2TH_MAX        7

unsigned int DrvPMU_VDD15Trim(void);
unsigned int DrvPMU_VDDA_Voltage(unsigned int uVoltage);
unsigned int DrvPMU_VDDA_LDO_Ctrl(unsigned int uCtrl);
void DrvPMU_BandgapEnable(void);
void DrvPMU_BandgapDisable(void);
void DrvPMU_REFO_Enable(void);
void DrvPMU_REFO_Disable(void);
unsigned int DrvPMU_AnalogGround(unsigned int uAG);
unsigned int DrvPMU_LDO_LowPower(unsigned int uLP);
unsigned int DrvPMU_GetLVDO(void);
void DrvPMU_EnableENLVD (void);
void DrvPMU_DisableENLVD (void);
unsigned int DrvPMU_SetLVDS(E_LVD_LVDS uVoltage);
unsigned int DrvPMU_SetLVDVS(unsigned char uLVDVS);
unsigned int DrvPMU_SetLVD12(unsigned char uLVD12);
unsigned int DrvPMU_LVDIntTriMode(E_LVDO_Status uLVDITT);
void DrvPMU_EnableLVDInt(void);
void DrvPMU_DisableLVDInt(void);
void DrvPMU_ClrLVDIF(void);
void DrvPMU_EnableBOR2 (unsigned char utime);
void DrvPMU_DisableBOR2 (void);
unsigned int DrvPMU_BOR2_Mode(E_BOR2_Mode uBOR2S);
unsigned int DrvPMU_BOR2_DetectVol(E_BOR2_BOR2TH uBOR2TH);
unsigned int DrvPMU_ReadBOR2Status(void);
#endif /* DRVPMU_H_ */
